getKEGG <- function(maps = "all",organism = "",kegg.xml.location = "",url = "ftp://ftp.genome.jp/pub/kegg/xml/kgml/metabolic/organisms") {
  mergedKEGG <- NULL
  hasXML <- require(XML)
  if (hasXML == FALSE) stop("To allow parsing of the KGML files please install the XML package.")

  curl <- NULL
  if (kegg.xml.location == "") {
    if (require(RCurl)) {
        cat("\nOpening connection to:",paste(url,"/",organism,"/",sep = ""))
        curl <- getCurlHandle()
    } else stop("To enable downloading of the KGML files please install the RCurl package.")
  }

  # Process the file names so that everything is expected
  if (maps[1] == "all") {
    # get all kegg pathway maps on the ftp site
    if (kegg.xml.location == "") maps <- getKEGGFileNames(organism,url = url)
    else {
      # get all kegg pathway maps in a local directory
      maps <- list.files(kegg.xml.location)
      maps <- grep(maps,pattern = ".xml",value = TRUE)
    }
    # remove the map corresponding to complete metabolism because we use the pathway groups in the annotations
    maps <- maps[-agrep("01100",maps)]
  } else {
    # ensure the specified maps have a xml extension
    idx <- grep(maps,pattern = ".xml")
    if (length(idx) != length(maps)) {
      if (length(idx) > 0) maps[-idx] <- paste(maps[-idx],".xml",sep = "")
      else maps <- paste(maps,".xml",sep = "")
    }
  }

  cat("\nProcessing KEGG XML files")
  # get the XML files
  for (m in maps) {
    KEGGxml <- NULL

    if (kegg.xml.location == "") {   
      cat("\n  Downloading KEGG XML map:",m)
      xmlTxt <- getURL(paste(url,"/",organism,"/",m,sep = ""),curl = curl)
      KEGGxml <- xmlTreeParse(xmlTxt)
    } else {
      cat("\n  Loading KEGG XML map:",m)
      KEGGxml <- xmlTreeParse(paste(kegg.xml.location,"/",m,sep = ""))
    }

    cat("\n  Converting XML to compound network")
    m.network <- getKEGGInfo(KEGGxml)

    if (!is.null(m.network)) {
      cat("\n  Merging into the overall compound network")
      if (is.null(mergedKEGG)) {
        mergedKEGG <- m.network
      } else {
        # Merge the compound network
        mergedKEGG$edge.list <- rbind(mergedKEGG$edge.list,m.network$edge.list)
        idx <- which(duplicated(mergedKEGG$edge.list))
        if (length(idx) > 0) mergedKEGG$edge.list <- mergedKEGG$edge.list[-idx,]

        # Merge the reaction list
        idx <- which(names(m.network$reaction.info) %in% names(mergedKEGG$reaction.info))
        if (length(idx) > 0) {
          for (r in idx) {
            g1 <- mergedKEGG$reaction.info[[ names(m.network$reaction.info)[r] ]]
            g2 <- m.network$reaction.info[[r]]
            g <- unique(c(g1,g2)) # if a reaction occurs in both network, take the gene set union
            mergedKEGG$reaction.info[[ names(m.network$reaction.info)[r] ]] <- g
          }
          mergedKEGG$reaction.info <- c(mergedKEGG$reaction.info,m.network$reaction.info[-idx])
        } else mergedKEGG$reaction.info <- c(mergedKEGG$reaction.info,m.network$reaction.info)
        mergedKEGG$details <- rbind(mergedKEGG$details,m.network$details)
      }
    }
  }

   return(mergedKEGG)
}

getKEGGFileNames <- function(organism,url) {
  kfiles <- c()

  organism.url <- paste(url,"/",organism,"/",sep = "")
  cat("\nGetting KEGG maps from",organism.url)
  
  kegginfo <- getURL(organism.url)

  kfiles <- strsplit(strsplit(kegginfo,"\n")[[1]]," ")
  kfiles <- unlist(sapply(kfiles,grep,pattern = ".xml",value = TRUE))

  return(kfiles)
}

# Takes the location and code of a KEGG XML file and finds a reaction->compound edge list and extracts
# which genes belong to each reaction
getKEGGInfo <- function(KEGG.XML) {
	pathway <- KEGG.XML$doc$children$pathway
	pathtitle <- xmlGetAttr(pathway,"title")
    pathcode <- strsplit(xmlGetAttr(pathway,"name"),":")[[1]][2]
    if (is.null(pathtitle)) pathtitle <- xmlGetAttr(pathway,"name")

	reaction.info <- list()
	edges <- data.frame(from = c(),to = c(),label = c(),pathway = c(),stringsAsFactors = FALSE)

	r.indx <- which(names(pathway) %in% "reaction")
	e.indx <- which(names(pathway) %in% "entry")
	if (length(r.indx) == 0) {
        cat("\n    !!!WARNING: No reactions found in pathway map!!!")
		return(NULL)
    }

	for (reaction in r.indx) {
        all.r.names <- strsplit(xmlGetAttr(pathway[[reaction]],"name")," ")[[1]]
        all.r.names <- all.r.names[all.r.names != ""]
        # One compound-compound transfer can occur by more than one reaction
        for (r.name in all.r.names) {
            disp.r.name <- strsplit(r.name,split = ":")[[1]][2]

            # First look to see if the new reaction has any valid genes
            reaction.has.genes <- FALSE
            for (entry in e.indx) {
                e.reaction <- xmlGetAttr(pathway[[entry]],"reaction")      
                if (!is.null(e.reaction)) { 
                    e.reaction <- strsplit(e.reaction,split = " ")[[1]]
                    e.reaction <- e.reaction[e.reaction != ""]
                    if (r.name %in% e.reaction) { 
                        reaction.genes <- strsplit(xmlGetAttr(pathway[[entry]],"name"),split = " ")[[1]]
                        reaction.genes <- reaction.genes[reaction.genes != ""] # remove whitespace genes
                        reaction.genes <- strsplit(toupper(reaction.genes),":")
                        
                        # exclude all KEGG ortholog elements as these wont be found in a microarray
                        ko.genes <- which(sapply(reaction.genes,"[[",1) == "KO") 
                        if (length(ko.genes) > 0) reaction.genes <- reaction.genes[-ko.genes]
                        reaction.genes <- sapply(reaction.genes,"[[",2)

                        if (length(reaction.genes) > 0) {
                            reaction.has.genes <- TRUE
                            if (disp.r.name %in% names(reaction.info)) {
                                reaction.info[[disp.r.name]] <-    
                                    c(reaction.info[[disp.r.name]],reaction.genes)
                            } else {
                                reaction.info <- c(reaction.info,list(reaction.genes))
                                names(reaction.info)[length(reaction.info)] <- disp.r.name
                            }
                        }
                    }
                }
            }
            
            # if the reaction has genes then add it to the network
            if (reaction.has.genes) {
                # add edge to compound-compound network labelled by reaction
                r.type <- xmlGetAttr(pathway[[reaction]],"type")
                
                r.from <- which(names(pathway[[reaction]]) %in% "substrate")
                r.to <- which(names(pathway[[reaction]]) %in% "product")
                for (sub in r.from) {
                    r.from.name <- strsplit(xmlGetAttr(pathway[[reaction]][[sub]],"name"),split = ":")[[1]][2]
                    for (prod in r.to) {
                        r.to.name <- strsplit(xmlGetAttr(pathway[[reaction]][[prod]],"name"),split = ":")[[1]][2]
      
                        edges <- rbind(edges,
                            data.frame(from = r.from.name,to = r.to.name,label = disp.r.name,pathway = pathtitle,
								   stringsAsFactors = FALSE))

                        if (r.type == "reversible") 
                            edges <- rbind(edges,
                                data.frame(from = r.to.name,to = r.from.name,label = disp.r.name,pathway = pathtitle,
									  stringsAsFactors = FALSE))
                    }
                } 
            }
		}
	}

	return(list(edge.list = edges,reaction.info = reaction.info,
                details = data.frame(code=pathcode,title = pathtitle,stringsAsFactors = FALSE)))
}

# take the output of kegg.info and make it a gene network for ichigaku's model.
computeGeneNetwork <- function(kegg.obj,start = NULL,end = NULL) {
    compoundNetwork <- kegg.obj[[1]]
    reactionList <- kegg.obj[[2]]

    cat("\nConverting compound/reaction network to gene/compound network")

    # Gene label = GENE_NAME:CFROM:CTO;REACTION:PATHWAY
    # this unfortunately is the most concise descriptor for every possible node in KEGG such that
    # broken pathways done occur.
    pstfunc <- function(i,compoundNetwork,reactionList) {
        gl <- reactionList[[compoundNetwork[i,]$label]]
        gl <- paste(gl,
                compoundNetwork[i,]$from,
                compoundNetwork[i,]$to,
                compoundNetwork[i,]$label,
                compoundNetwork[i,]$pathway,sep = ":")
        return(gl)
    }
    genelist <- lapply(1:nrow(compoundNetwork),pstfunc,compoundNetwork,reactionList)
    genelist <- unique(unlist(genelist))
    genelist <- c("s",genelist,"t")

    #define all start and end compounds
    all.compounds <- unique(c(kegg.obj$edge[,1],kegg.obj$edge[,2]))
    if (is.null(start)) start <- all.compounds[which(all.compounds %in% kegg.obj$edge$to == FALSE)]
    if (is.null(end)) end <- all.compounds[which(all.compounds %in% kegg.obj$edge$from == FALSE)]
    if (length(start) > 0 & length(end) > 0) {
		# for each start compound, make a pseudo edges of the from:
		# edge(from = "s", to = "start compound i",by = "pseudo reaction 1")
		# pseudo_reaction_1 just contains gene "s"
		start.edges <- NULL
		for (si in 1:length(start)) {
			sc <- start[si]
			srname <- paste("Start",sc,sep = "-")

			idx <- which(compoundNetwork$from %in% sc)
			rlabels <- unique(compoundNetwork[idx,]$label)
			s.edges <- data.frame(from = "s",
					to = sc,
					label = srname,
					pathway = "start",
					stringsAsFactors = FALSE)
			reactionList[[length(reactionList) + 1]] <- c("s")
			names(reactionList)[length(reactionList)] <- srname		

			if (is.null(start.edges)) start.edges <- s.edges
			else start.edges <- rbind(start.edges,s.edges)
		}
   
		# for each end compound, make a pseudo edges of the from:
		# edge(from = "end compound i", to = "t",by = "pseudo reaction 1")
		# pseudo_reaction_1 just contains gene "t"
		end.edges <- NULL
		for (ei in 1:length(end)) {
			ec <- end[ei]
			ername <- paste("End",ec,sep = "-")

			idx <- which(compoundNetwork$to %in% ec)
			rlabels <- unique(compoundNetwork[idx,]$label)
			e.edges <- data.frame(from = ec,
								  to = "t",
								  label=ername,
								  pathway = "end",
								  stringsAsFactors = FALSE)
			reactionList[[length(reactionList) + 1]] <- c("t")
			names(reactionList)[length(reactionList)] <- ername		
	
			if (is.null(start.edges)) end.edges <- e.edges
			else end.edges <- rbind(end.edges,e.edges)
		}
		# Make the network
		compoundNetwork <- rbind(start.edges,compoundNetwork,end.edges)
	} else stop("Must define a set of start and end compounds")

    # Convert a compound network into a gene network *brackets indicate the edge label
	# Compound network: C1 --> (g1) --> C2 --> (g2) --> C3
	# Gene network: g1 --> (C1:C2:C3) --> g2
	geneNetwork <- NULL
	for (i in 1:nrow(compoundNetwork)) {
		# process the first set start compound -> middle
		cfrom <- as.character(compoundNetwork[i,]$from)
		rfrom <- as.character(compoundNetwork[i,]$label) # reactions joining start and middle compounds
        gfrom <- c(1)
        if (cfrom != "s") {
            gl <- paste(reactionList[[rfrom]],
                    cfrom,
                    compoundNetwork[i,]$to,
                    rfrom,
                    compoundNetwork[i,]$pathway,sep = ":")
            gfrom <- which(genelist %in% gl)  # index of genes joining the start and middle compounds
        }

		# process the next step, middle compound -> next set of compounds
		middle <- as.character(compoundNetwork[i,]$to) # middle compound - also the edge label
		path <- as.character(compoundNetwork[i,]$pathway)
		edgeIdx <- which(compoundNetwork$from == middle) # indices reactions steming from the middle compound
		if (length(edgeIdx) > 0) { # if the middle compound is not a terminating compound
			zcomps <- compoundNetwork[edgeIdx,]
			idx <- which(zcomps$to == cfrom) # cannot have a self loop from cfrom -> middle -> cfrom
			if (length(idx) > 0) zcomps <- zcomps[-idx,] 
			
			if (nrow(zcomps) > 0) {
               edges <- NULL
               for (j in 1:nrow(zcomps)) {
                    rto <- as.character(zcomps[j,]$label) # reactions joining middle and end compounds
                    path <- zcomps[j,]$pathway
                    cend <- zcomps[j,]$to
                    gto <- length(genelist)
                    if (cend != "t") {
                        gl <- paste(reactionList[[rto]],
                                zcomps[j,]$from,
                                zcomps[j,]$to,
                                zcomps[j,]$label,
                                zcomps[j,]$pathway,sep = ":")
                         gto <- which(genelist %in% gl) # get the gene indices of gto
                    }
                   
                    # make the edges and append to the geneNetwork
                    # interaction always throws a warning, all this does is
                    # get all combinations of gfrom and gto which may be of different sizes
                    eds <- suppressWarnings(levels(interaction(gfrom,gto))) # create all edge combinations of gfrom -> gto
                    eds <- lapply(strsplit(eds,".",fixed = TRUE),as.numeric)
                    eds <- data.frame(t(do.call("cbind",eds)))
                    idx <- which(eds[,1] == eds[,2]) #remove edges from one gene to itself
                    if (length(idx) > 0) eds <- eds[-idx,]

                    if (nrow(eds) > 0) {
                        # Each edge is labelled by C_from:C_middle:C_end and pathway (C = compound)
                        # Each gene is labelled by Gene:Reaction
                        # So the edge information is complete
                        label <- paste(c(cfrom,middle,cend),collapse = ":")
                        eds <- data.frame(cbind(eds,label,path))
                        names(eds) <- c("from","to","label","pathway")
                        if (is.null(edges)) edges <- eds
                        else edges <- rbind(edges,eds)
                    }
               }

               # if there are any edges left then add them to the network
               if (!is.null(edges)) {                  
                    if (is.null(geneNetwork)) geneNetwork <- edges
                    else geneNetwork <- rbind(geneNetwork,edges)
               }
			}
		}
	}

	idx <- which(duplicated(geneNetwork))
	if (length(idx) > 0) geneNetwork <- geneNetwork[-idx,]
	geneNetwork <- with(geneNetwork,geneNetwork[order(from,to),])
	dimnames(geneNetwork)[[1]] <- 1:nrow(geneNetwork)
	geneNetwork <- list(nodes = genelist,edges = geneNetwork,start = start,end = end)
	geneNetwork$edges$label <- as.character(geneNetwork$edges$label)
	geneNetwork$edges$pathway <- as.character(geneNetwork$edges$pathway)

	return(geneNetwork)
}

assignEdgeWeights <- function(microarray,geneNetwork,y = NULL,corRepeats = 100) {
	gn.r <- geneNetwork$nodes
	gn <- sapply(strsplit(gn.r,":"),"[[",1) #break the gene:reaction edge labels

	not.included.genes <-  which((gn %in% names(microarray)) == FALSE)
	p.data <- data.frame(matrix(0,nrow = nrow(microarray),ncol = length(not.included.genes)))
	names(p.data) <- gn[not.included.genes]
	p.microarray <- cbind(microarray,p.data)
  
    miss.genes <- unique(names(p.data))
    miss.genes <- miss.genes[-which(miss.genes %in% c("s","t"))]
    if (length(miss.genes) > 0) {
        cat("\nWarning: Could not find the following genes in the microarray")
        cat("\n",miss.genes)
    }
  
	all.genes <- p.microarray[gn] # extract all relevant genes
	names(all.genes) <- gn.r # relabel by the gene:reaction labels

    # correlation function
    compCor <- function(rowSamples,all.genes,from,to,weight,sindx,tindx,samegene) {
        ag <- all.genes[rowSamples,]
        all.cors <- .C("corEdgeWeights",
            as.double(as.matrix(ag)),
            as.integer(from),
            as.integer(to),
            as.integer(samegene),
            weight = as.double(weight),
            as.integer(length(from)),
            as.integer(nrow(ag)),
            as.integer(sindx),
            as.integer(tindx),NAOK = TRUE)
        return(all.cors$weight)(all.cors$weight)
    }

    # Compute all gene-gene correlations
    sindx <- which(geneNetwork[[1]] == "s") - 1
    tindx <- which(geneNetwork[[1]] == "t") - 1 
    from <- geneNetwork$edges[,1] - 1
    to <- geneNetwork$edges[,2] - 1

    # find all same gene edges
    gt <- sapply(strsplit(geneNetwork$nodes[geneNetwork$edges$to],":"),"[[",1)
    gf <- sapply(strsplit(geneNetwork$nodes[geneNetwork$edges$from],":"),"[[",1)
    idx <- which(gt == gf)
    samegene <- as.integer(gt == gf)
    
    edge.weights <- NULL
    if (is.null(y)) {
        cat("\nAssigning edge weights")      
        if (corRepeats > 1) {
            cat("\nTaking the median of",corRepeats,"Bootstrapped correlations - please wait a bit...")
            bsize <- ifelse(nrow(all.genes) < 10,10,nrow(all.genes))
            w.list <- lapply(1:corRepeats,function(i) sample(1:nrow(all.genes),bsize,replace = TRUE))
            c.list <- lapply(w.list,compCor,all.genes,from,to,double(length(from)),sindx,tindx,samegene)
            c.list <- do.call(cbind,c.list)
            edge.weights <- apply(c.list,1,median,na.rm = TRUE) 
        } else edge.weights <- compCor(1:nrow(all.genes),all.genes,from,to,double(length(from)),sindx,tindx,samegene)
        edge.weights <- data.frame(edge.weights)
        names(edge.weights) <- "cor"
    } else {
        for (yl in 1:nlevels(y)) {  
            data <- all.genes[y == levels(y)[yl],]
            cat("\nAssigning edge weights for label",levels(y)[yl]) 
            ed <- c()
            if (corRepeats > 1) {
                cat("\nTaking the median of",corRepeats,"Bootstrapped correlations")
                bsize <- ifelse(nrow(data) < 10,10,nrow(all.genes))
                w.list <- lapply(1:corRepeats,function(i) sample(1:nrow(data),bsize,replace = TRUE))
                c.list <- lapply(w.list,compCor,data,from,to,double(length(from)),sindx,tindx,samegene)
                c.list <- do.call(cbind,c.list)
                ed <- apply(c.list,1,median,na.rm = TRUE) 
            } else ed <- compCor(1:nrow(data),data,from,to,double(length(from)),sindx,tindx,samegene)
            if (is.null(edge.weights)) edge.weights <- data.frame(ed)
            else edge.weights <- cbind(edge.weights,data.frame(ed))
        }
        names(edge.weights) <- paste("cor:",levels(y),sep = "")
    }
	
	return(edge.weights)
}

pathRanker <- function(geneNetwork,edge.weights,K=10,minPathSize = 1,sameGenePenalty = 1/10,normalize = TRUE) {
    gn <- geneNetwork$edge
    gn$label <- paste(gn$label,gn$pathway,sep = ":")

    gt <- sapply(strsplit(geneNetwork$nodes[geneNetwork$edges$to],":"),"[[",1)
    gf <- sapply(strsplit(geneNetwork$nodes[geneNetwork$edges$from],":"),"[[",1)
    idx <- which(gt == gf)

    edge.probs <- edge.weights
    for (i in 1:ncol(edge.weights)) {
        edge.probs[,i] <- ecdf(edge.weights[,i])(edge.weights[,i])
        if (length(idx) > 0) edge.probs[,i][idx] <- sameGenePenalty
    }
    if (ncol(edge.probs) > 1 & normalize == TRUE) {
        cat("\nNormalizing edges by response label")
        edge.probs <- edge.probs / rowSums(edge.probs)
    }
    edge.probs <- -log(edge.probs)    

    if (ncol(edge.weights) > 1) labels <- sapply(strsplit(names(edge.weights),":"),"[[",2)
    zret <- list()
    for (i in 1:ncol(edge.probs)) {
        if (ncol(edge.probs) > 1) cat("\nExtracting the",K,"most probable paths for",labels[i])
        else cat("\nExtracting the",K,"most probable paths")
       
        # min path size is increased by 2 to include start and end compounds!!!
        ps <- .Call("pathranker",geneNetwork$nodes,gn,edge.probs[,i],K=K,minpathsize = minPathSize+2)
        idx <- which(sapply(ps,is.null))
        if (length(idx)>0) ps <- ps[-idx]

        if (ncol(edge.probs) > 1) zret[[i]] <- ps
        else zret <- ps
    }

    if (ncol(edge.weights) > 1) {
        names(zret) <- labels
        names(edge.probs) <- paste("prob",labels,sep = ":")
    } else names(edge.probs) <- "prob"

    return(list(paths = zret,edge.weights = edge.probs))
}


plotPathRanker <- function(PRpaths,tol = NULL) {
  if (!is.null(names(PRpaths$paths))) paths <- unlist(PRpaths$paths,recursive = FALSE)
  else paths <- PRpaths$paths

  g <- unique(unlist(lapply(paths,"[[","genes")))
  tg <- g[-which(g %in% c("s","t"))]
  idx <- sort(sapply(strsplit(tg,":"),"[[",5),index.return = TRUE)$ix
  g <- c("s",tg[idx],"t")

  makeBin <- function(pathGenes,allGenes) return(as.numeric(allGenes %in% pathGenes$genes))
  binpaths <- data.frame(t(sapply(paths,makeBin,allGenes =g)))
  gene.ess <- colMeans(binpaths)
  binpaths[binpaths == 0] <- NA
  binpaths <- binpaths[nrow(binpaths):1,]
  if (!is.null(tol)) {
    zp <- which(gene.ess < tol)
    if (length(zp) > 0) {binpaths <- binpaths[-zp]; g <- g[-zp]; gene.ess <- gene.ess[-zp]}
  }
   
  mpar <- par()$mar

  layout(matrix(c(1,3,2,8,4,7,5,6),2,4),
        heights = c(0.4,0.6), #,0.5,0.5), #,0.5,0.5,0.5,0.5),
        widths =  c(0.75,0.1,0.1,0.05),TRUE)

  par(mar = c(0,6,1,0),xaxs = "i")
  image(x = 1:ncol(binpaths),y = 1:nrow(binpaths),as.matrix(t(binpaths)),axes = F,
      ylab = "",xlab = "",xaxs = "i",yaxs = "i",col = rainbow(4))
  ptck <- seq(1,nrow(binpaths),length.out = 10)
  axis(2,at = ptck,round(rev(ptck)),las = 2)
  mtext("All Paths\nSorted By Distance Covered",2,line = 3.5)
  if (!is.null(names(PRpaths$paths))) {
    pl <- sapply(PRpaths$paths,length)
    abline(h = cumsum(pl),col = 1,lwd = 2)
  }
  
  d <- exp(-sapply(paths,"[[","distance"))
  par(mar = c(0,0,1,0))
  plot(y = nrow(binpaths):1,x = d,axes = FALSE,xlab = "",ylab = "",type = "l",
      col = 2,lwd = 3,xaxs = "i",yaxs = "i",xlim = c(-max(d)/5,max(d)))
  m1 <- 0; m2 <- range(d)[2]
  axis(1,at = seq(m1,m2,(m2 - m1)/3),round(seq(m1,m2,(m2 - m1)/3),2),las = 2)
  if (!is.null(names(PRpaths$paths))) {
    pl <- sapply(PRpaths$paths,length)
    abline(h = cumsum(pl),col = 1,lwd = 2)
  }
  abline(v = 0,lwd = 2,col = 1)
  mtext("Path\nProbability",1,5,cex = 0.8,las = 2)

  palette("default")
  gc <- strsplit(g,":")[-c(1,length(g))]
  frt <- c("",paste(sapply(gc,"[[",2),sapply(gc,"[[",4),sapply(gc,"[[",3),sep = "-"),"")
  gn <- c("s",sapply(gc,"[[",1),"t")
  pp <- c(0,as.numeric(as.factor(sapply(gc,"[[",5))),0)

  par(mar = c(30,6,1,0))
  barcols <- as.numeric(cut(gene.ess,5))
  ht <- barplot(gene.ess,horiz = FALSE,axes = FALSE,xaxs = "i",las = 2,col = pp %% 6 + 1,names.arg = "")
 
  mtext(gn,side = 1,at = ht,cex = 0.6,las =2,line = 0.5,col = pp %% 6 + 1)
  mtext(frt,side = 1,at = ht,cex = 0.6,las = 2,line = 4.5,col = pp %% 6 + 1)
  pd <- cbind(ht[-c(1,length(ht))],pp[-c(1,length(pp))])
  pht <- aggregate(pd,list(sapply(gc,"[[",5)),median)
  mtext(pht[,1],side = 1,at = pht[,2],cex = 0.6,las = 2,line = 15, 
        col = as.numeric(as.factor(pht[,3])) %% 6 + 1)
  axis(2,at = c(0,0.2,0.4,0.6,0.8,1),labels = c(0,0.2,0.4,0.6,0.8,1),las = 2)
  mtext("Gene\nProbability",2,line = 3.5)

  plen <- as.numeric(sapply(lapply(paths,"[[","genes"),"length") - 2)
  score <- as.numeric(sapply(paths,"[[","distance"))
  pvalue <- 1- pgamma(score,plen-1,lower = F,log = F)
  par(mar = c(0,0,1,0))
  plot(y = nrow(binpaths):1,x = pvalue,axes = FALSE,xlab = "",ylab = "",type = "l",
      col = 2,lwd = 3,xaxs = "i",yaxs = "i",xlim = c(-max(pvalue)/5,max(pvalue)))
  m1 <- 0; m2 <- range(pvalue)[2]
  axis(1,at = seq(m1,m2,(m2 - m1)/3),round(seq(m1,m2,(m2 - m1)/3),2),las = 2)
  if (!is.null(names(PRpaths$paths))) {
    pl <- sapply(PRpaths$paths,length)
    abline(h = cumsum(pl),col = 1,lwd = 2)
  }
  abline(v = 0,lwd = 2,col = 1)
  mtext("Path\nP-Value",1,5,cex = 0.8,las = 2)

  if (!is.null(names(PRpaths$paths))) {
    par(mar = c(0,0,1,0))
    plot(NA,axes = FALSE,xlab = "",ylab = "",type = "l",
       col = 2,lwd = 3,xaxs = "i",yaxs = "i",xlim = c(0,1),ylim = c(1,nrow(binpaths)))
    pl <- sapply(PRpaths$paths,length)
    abline(h = cumsum(pl),col = 1,lwd = 2)
    text(x = 0.5,y = rev(cumsum(pl) - 0.5*pl),names(PRpaths$paths),srt = 90)
  }

  layout(1)
  par(mar = mpar)
}
