\name{pathClassifier}
\alias{pathClassifier}
\title{HME3M Markov pathway classifier.}
\description{HME3M Markov pathway classifier.}

\usage{
pobj <- pathClassifier(paths,target.class,M,alpha=1,lambda=2,hme3miter = 100,plriter = 1,init = "random")
}
\arguments{
\item{paths}{The training paths computed by \code{\link{pathsToBinary}}}
\item{target.class}{The label of the targe class to be classified.  This label must be present as a label within the \code{paths\$y} object}
\item{M}{Number of components within the paths to be extracted.}
\item{alpha}{The PLR learning rate. (between 0 and 1)}
\item{lambda}{The PLR regularization parameter. (between 0 and 2}
\item{hme3miter}{Maximum number of HME3M iterations.  It will stop when likelihood change is < 0.001.}
\item{plriter}{Maximum number of PLR iteractions. It will stop when likelihood change is < 0.001.}
\item{init}{Specify whether to initialize the HME3M responsibilities with the 3M model - random is recommended}
}
\value{
A list with the following values
\item{h}{A dataframe with the EM responsibilities.}
\item{theta}{A dataframe with the Markov parameters for each component.}
\item{beta}{A dataframe with the PLR coefficients for each component.}
\item{proportions}{The probability of each HME3M component.}
\item{posterior.probs}{The HME3M posterior probability.}
\item{likelihood}{The likelihood convergence history.}
\item{plrplr}{The posterior predictions from each components PLR model.}
\item{path.probabilities}{The 3M probabilities for each path belonging to each component.}
\item{params}{The parameters used to build the model.}
\item{y}{The binary response variable used by HME3M. A 1 indicates the location of the target.class labels in \code{paths\$y}}
\item{perf}{The training set ROC curve AUC.}
\item{label}{The HME3M predicted label for each path.}
\item{component}{The HME3M component assignment for each path.}
}
\details{
Take care with selection of lambda and alpha - make sure you check that the likelihood is always increasing.
}
\references{
1. Hancock, Timothy, and Mamitsuka, Hiroshi: A Markov Classification Model for Metabolic Pathways, Workshop on Algorithms in Bioinformatics (WABI) , 2009

2. Hancock, Timothy, and Mamitsuka, Hiroshi: A Markov Classification Model for Metabolic Pathways, Algorithms for Molecular Biology 2010
}
\seealso{
\code{\link{pathsToBinary}}, \code{\link{predictPathClassifier}}, \code{\link{plotPathClassifier}}, \code{\link{plotPathClassifierPath}}, \code{\link{plotComponents}}, \code{\link{extractPathNetwork}}, \code{\link{plotPathNetwork}}
}
\examples{
#########################################
## Prepare the microarray data
#########################################

# Organize some example microarray from the gaschYHS package
source("http://bioconductor.org/biocLite.R")
biocLite("gaschYHS")
library(gaschYHS)
data(gaschYHS)

microarray <- as.data.frame(t(exprs(gaschYHS)))

# Make a response to iosolate the "Stationary Phase" group.
stress.conditions <- rep("other",nrow(microarray))
stress.conditions[agrep("steady.state",rownames(microarray))] <- "Steady State"
stress.conditions <- as.factor(stress.conditions)

#########################################
## Get the KEGG network
#########################################
library(PathRanker)

# Download and process the KGML files for yeast into a 
# compound-compound network with reaction information
# if maps is left blank all found paths will be extracted.
# here we extract all pathways listed on the glycolysis KEGG pathway map
kobj <- getKEGG(maps = c("sce00010","sce00020","sce00030","sce00500",
                         "sce00620","sce00640"),organism = "sce")

# Convert the compound network into a gene network with start and end compounds
# Blank start and end compounds means we will use all valid start and end 
# compounds within the network
geneNetwork <- computeGeneNetwork(kobj)

# Compute bootstrapped correlations to be used as edge weights to 
# the geneNetwork object.
# ---> if there is not response variable simply remove the y parameter.
edge.weights <- assignEdgeWeights(microarray,geneNetwork,
                                  y = stress.conditions, corRepeats = 100)

#########################################
## Rank the pathways
#########################################

# Extract the top 1000 paths with:
# --> 9 genes or more 
# --> same gene penalty equal to a correlation of 0
# --> not normalized over the response labels
ypaths <- pathRanker(geneNetwork = geneNetwork,edge.weights = edge.weights,
        K = 1000,sameGenePenalty = 0,minPathSize = 5,normalize = FALSE)

# Plot all paths extracted
# --> set tol to filter out unimportant small genes.
plotPathRanker(ypaths)

#########################################
## Convert to a binary
#########################################

 #Convert the paths to a binary matrix
ybinpaths <- pathsToBinary(ypaths)

# To look up any path with the 'ypaths' object based on its location 
# within the binary matrix use the pidx element within the returned 
#ybinpaths object.  
p <- ypaths[[ybinpaths$pidx[120,]]]

#########################################
## Cluster the pathways
#########################################

pclust <- pathCluster(paths = ybinpaths,M = 2)
# Contingency table of clustering performance
table(ybinpaths$y,pclust$label)

# just an example of how to predict cluster membership
pclust.pred <- predictPathCluster(pclust,ybinpaths$paths)

# Plot the components identified by 3M
# --> set tol to filter out unimportant small genes.
# --> for easier interpretation the cluster option 
#     uses heirarchical clustering to group similar components.
plotComponents(pclust)

# Plot a specific path idetified by 3M
# --> set tol to filter out unimportant small genes.
plotPathClusterPath(paths = ybinpaths,obj = pclust,m = 1)

# Extract the compound network for 3M cluster 1
pnet <- extractPathNetwork(ypaths,pathweights = pclust$h[,1],
                           tol = 0.5,feature = "compound")

# Plot the network using Rgraphviz
plotPathNetwork(pnet)

#########################################
## Classify known path labels
#########################################

pclass <- pathClassifier(paths = ybinpaths,
                         target.class = "Steady State",M = 2)
# Contingency table of classification performance
table(ybinpaths$y,pclass$label)

# a plot of the performance of the HME3M model
plotPathClassifier(pclass)

# just an example of how to predict cluster membership
pclass.pred <- predictPathCluster(pclass,ybinpaths$paths)

# Plot the components identified by HME3M
# --> set tol to filter out unimportant small genes.
# --> for easier interpretation the cluster option 
#     uses heirarchical clustering to group similar components.
plotComponents(pclass)

# Plot a specific path idetified by HME3M
# --> set tol to filter out unimportant small genes.
plotPathClassifierPath(paths = ybinpaths,obj = pclass,m = 1)

# Extract the pathway network with edges weighted by the 
# posterior probabilities for classifying the response label.
# To extract the network of all paths belonging specific group 
# use a binary pathweights where 1 is the position of group.
pnet <- extractPathNetwork(ypaths,pathweights = pclass$post,tol = 0.5,feature = "pathway")

# Plot the network using Rgraphviz
plotPathNetwork(pnet)
}
