\name{pathCluster}
\alias{pathCluster}
\title{3M Markov mixture model for clustering pathways.}
\description{3M Markov mixture model for clustering pathways.}

\usage{
pathCluster(paths,M,iter = 1000)
}
\arguments{
\item{paths}{The training paths computed by \code{\link{pathsToBinary}}.}
\item{M}{The number of clusters.}
\item{iter}{The maximum number of EM iterations.}
}
\value{
A list with the following items:
\item{h}{The posterior probabilities that each path belongs to each cluster.}
\item{labels}{The cluster membership labels.}
\item{theta}{The probabilities of each gene for each cluster.}
\item{proportions}{The mixing proportions of each path.}
\item{likelihood}{The likelihood convergence history.}
\item{params}{The specific parameters used.}
}
\references{
Mamitsuka, H., Okuno, Y., and Yamaguchi, A. 2003. Mining biologically active patterns in metabolic pathways using microarray expression profiles. SIGKDD Explor. News l. 5, 2 (Dec. 2003), 113-121. 
}
\seealso{
\code{\link{pathRanker}}, \code{\link{pathsToBinary}}, \code{\link{predictPathCluster}}, \code{\link{plotPathClusterPath}},
\code{\link{plotComponents}}, \code{\link{extractPathNetwork}} , \code{\link{plotPathNetwork}}
}
\examples{
#########################################
## Prepare the microarray data
#########################################

# Organize some example microarray from the gaschYHS package
source("http://bioconductor.org/biocLite.R")
biocLite("gaschYHS")
library(gaschYHS)
data(gaschYHS)

microarray <- as.data.frame(t(exprs(gaschYHS)))

# Make a response to iosolate the "Stationary Phase" group.
stress.conditions <- rep("other",nrow(microarray))
stress.conditions[agrep("steady.state",rownames(microarray))] <- "Steady State"
stress.conditions <- as.factor(stress.conditions)

#########################################
## Get the KEGG network
#########################################
library(PathRanker)

# Download and process the KGML files for yeast into a 
# compound-compound network with reaction information
# if maps is left blank all found paths will be extracted.
# here we extract all pathways listed on the glycolysis KEGG pathway map
kobj <- getKEGG(maps = c("sce00010","sce00020","sce00030","sce00500",
                         "sce00620","sce00640"),organism = "sce")

# Convert the compound network into a gene network with start and end compounds
# Blank start and end compounds means we will use all valid start and end 
# compounds within the network
geneNetwork <- computeGeneNetwork(kobj)

# Compute bootstrapped correlations to be used as edge weights to 
# the geneNetwork object.
# ---> if there is not response variable simply remove the y parameter.
edge.weights <- assignEdgeWeights(microarray,geneNetwork,
                                  y = stress.conditions, corRepeats = 100)

#########################################
## Rank the pathways
#########################################

# Extract the top 1000 paths with:
# --> 9 genes or more 
# --> same gene penalty equal to a correlation of 0
# --> not normalized over the response labels
ypaths <- pathRanker(geneNetwork = geneNetwork,edge.weights = edge.weights,
        K = 1000,sameGenePenalty = 0,minPathSize = 5,normalize = FALSE)

# Plot all paths extracted
# --> set tol to filter out unimportant small genes.
plotPathRanker(ypaths)

#########################################
## Convert to a binary
#########################################

 #Convert the paths to a binary matrix
ybinpaths <- pathsToBinary(ypaths)

# To look up any path with the 'ypaths' object based on its location 
# within the binary matrix use the pidx element within the returned 
#ybinpaths object.  
p <- ypaths[[ybinpaths$pidx[120,]]]

#########################################
## Cluster the pathways
#########################################

pclust <- pathCluster(paths = ybinpaths,M = 2)
# Contingency table of clustering performance
table(ybinpaths$y,pclust$label)

# just an example of how to predict cluster membership
pclust.pred <- predictPathCluster(pclust,ybinpaths$paths)

# Plot the components identified by 3M
# --> set tol to filter out unimportant small genes.
# --> for easier interpretation the cluster option 
#     uses heirarchical clustering to group similar components.
plotComponents(pclust)

# Plot a specific path idetified by 3M
# --> set tol to filter out unimportant small genes.
plotPathClusterPath(paths = ybinpaths,obj = pclust,m = 1)

# Extract the pathway network for 3M cluster 1
pnet <- extractPathNetwork(ypaths,pathweights = pclust$h[,1],
                           tol = 0.5,feature = "pathway")

# Plot the network using Rgraphviz
plotPathNetwork(pnet)
}


