\name{pathRanker}
\alias{pathRanker}
\title{Extracts the K most probable paths within a metabolic network.}
\description{Extracts the K most probable paths within a metabolic network.}

\usage{
ypaths <- pathRanker(geneNetwork,edge.weights,K=10,minPathSize = 1,sameGenePenalty = 1/10,normalize = TRUE) 
}
\arguments{
\item{geneNetwork}{A gene network with probabilitic edge list as given by \code{\link{assignEdgeWeights}}.}
\item{edge.weights}{A matrix of weights for each edge.  Each column in the matrix corresponds the label of a specified response (\code{\link{assignEdgeWeights}}).  An individual set of paths will be extracted using each set of columns.}
\item{K}{Maximum number of paths to extract.}
\item{minPathSize}{The minimum number of edges for each path.}
\item{sameGenePenalty}{The penalty for selecting the same gene in two neighboring reactions.}
\item{normalize}{Specify if you want to normalize the probabilistic edge weights before extracting the paths.}
}
\value{
A list of pathways where each path has the following items:
\item{gene}{The ordered sequence of genes visited along the path.}
\item{compounds}{The ordered sequence of compounds visited along the path.}
\item{weights}{The ordered sequence of the log(ECDF edge weights) along the path.}
\item{distance}{The sum of the log(ECDF edge weights) along each path.  (a sum of logs is a product)}
}
\details{
\code{pathRanker} finds the K most probable loopless paths given a set of weights assigned to the network in computed by \code{\link{computeGeneNetwork}}. 

A path is defined as a sequence of genes where each edge is labelled by the exact position within the metabolic network where it was observed.

Before the paths are ranked the edge weights are converted into probabilistic edge weights using the Empirical Cumulative Distribution (ECDF) over all edge weights.  This is called ECDF edge weight. The ECDF edge weight serves as a probabilistic rank of the most important gene-gene interactions.  The probabilistic nature of the ECDF edge weights allow for a significance test to determine if a path contains any functional structure or is simply a random walk.  The probability of a path is simily the product of all ECDF weights along the path.  This is computed as a sum of the logs of the ECDF edge weights.

A weight must be assigned to each edge. Bootstrapped Pearson correlation edge weights can be assigned to each edge by \code{\link{assignEdgeWeights}}. However the specification of the edge weight is flexible with the condition that increasing values indicate stronger relationships between genes.  See the structure of the matrix returned by \code{\link{assignEdgeWeights}} for the required data structure for the edge weights.

As the same gene is likely to occur in neighboring reactions which may result in the weight becoming 0 for that edge.  If too many occurrence of 0 edge weight occur the method will get caught in small zero-sum loops within the network.  To avoid this use the \code{sameGenePenalty} option which replaces all weights of edges connected with the same gene to a specified value.  The same gene penalty adjustment is performed on the ECDF edge weights so the appropriate parameter range is [0,1] where 0 is the lowest and penalizes the strongest and 1 is the highest and corresponds to no same gene penalty.
}
\references{
1. Hancock, T., Takigawa, I. and Mamitsuka, "Mining metabolic pathways through gene expression", (In Press).

2. Takigawa, I. and Mamitsuka, H. Probabilistic Path Ranking Based on Adjacent Pairwise Coexpression for Metabolic Transcripts Analysis.  Bioinformatics, 24 (2), 250-257, 2008.
}
\seealso{
\code{\link{getKEGG}}, \code{\link{computeGeneNetwork}}, \code{\link{assignEdgeWeights}},  \code{\link{plotPathRanker}}
}
\examples{
#########################################
## Prepare the microarray data
#########################################

# Organize some example microarray from the gaschYHS package
source("http://bioconductor.org/biocLite.R")
biocLite("gaschYHS")
library(gaschYHS)
data(gaschYHS)

microarray <- as.data.frame(t(exprs(gaschYHS)))

# Make a response to iosolate the "Stationary Phase" group.
stress.conditions <- rep("other",nrow(microarray))
stress.conditions[agrep("steady.state",rownames(microarray))] <- "Steady State"
stress.conditions <- as.factor(stress.conditions)

#########################################
## Get the KEGG network
#########################################
library(PathRanker)

# Download and process the KGML files for yeast into a 
# compound-compound network with reaction information
# if maps is left blank all found paths will be extracted.
# here we extract all pathways listed on the glycolysis KEGG pathway map
kobj <- getKEGG(maps = c("sce00010","sce00020","sce00030","sce00500",
                         "sce00620","sce00640"),organism = "sce")

# Convert the compound network into a gene network with start and end compounds
# Blank start and end compounds means we will use all valid start and end 
# compounds within the network
geneNetwork <- computeGeneNetwork(kobj)

# Compute bootstrapped correlations to be used as edge weights to 
# the geneNetwork object.
# ---> if there is not response variable simply remove the y parameter.
edge.weights <- assignEdgeWeights(microarray,geneNetwork,
                                  y = stress.conditions, corRepeats = 100)

#########################################
## Rank the pathways
#########################################

# Extract the top 1000 paths with:
# --> 9 genes or more 
# --> same gene penalty equal to a correlation of 0
# --> not normalized over the response labels
ypaths <- pathRanker(geneNetwork = geneNetwork,edge.weights = edge.weights,
        K = 1000,sameGenePenalty = 0,minPathSize = 5,normalize = FALSE)

# Plot all paths extracted
# --> set tol to filter out unimportant small genes.
plotPathRanker(ypaths)
}



